package cn.estsh.i3plus.core.apiservice.auth;

import cn.estsh.i3plus.platform.common.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.platform.common.util.CommonConstWords;
import cn.estsh.i3plus.pojo.platform.bean.SessionUser;
import cn.estsh.impp.framework.boot.util.ImppRedis;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.math.NumberUtils;
import org.apache.shiro.SecurityUtils;
import org.apache.shiro.authc.AuthenticationToken;
import org.apache.shiro.session.Session;
import org.apache.shiro.subject.Subject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;

/**
 * @Description : 授权会话工具类
 * @Reference :
 * @Author : alwaysfrin
 * @CreateDate : 2018-10-13 13:21
 * @Modify:
 **/
public class AuthUtil {
    public static final Logger LOGGER = LoggerFactory.getLogger(AuthUtil.class);

    /**
     * 缓存
     */
    @Resource(name="redisSession")
    private ImppRedis redisSession;

    //登陆，保存会话对象
    public static SessionUser login(AuthenticationToken token){
        Subject subject = SecurityUtils.getSubject();
        subject.login(token);

        SessionUser sessionUser = (SessionUser) subject.getPrincipal();
        //使用了redis，此处不需要防止
        if(sessionUser != null){
            //userType为CommonEnumUtil.USER_TYPE.xxx.getCode()
            setSessionUser(sessionUser);
        }

        return sessionUser;
    }

    /**
     * 退出，移除会话中对象
     */
    public static void logout(){
        Session session = SecurityUtils.getSubject().getSession(false);
        if(session != null) {
            session.removeAttribute(CommonConstWords.SESSION_USER);
            session.touch();
            session.stop();
        }
    }

    /**
     * 保存会话中用户
     * @param user
     * @return
     */
    public static void setSessionUser(SessionUser user){
        SecurityUtils.getSubject().getSession(true).setAttribute(CommonConstWords.SESSION_USER,user);
    }
    /**
     * 获取会话中用户
     * @return
     */
    public static SessionUser getSessionUser(){
        Session session = SecurityUtils.getSubject().getSession(false);
        if(session != null){
            return (SessionUser) session.getAttribute(CommonConstWords.SESSION_USER);
        }else{
            return null;
        }
    }

    /**
     * 保存会话中对象
     * @param key   键
     * @param obj   值
     * @return
     */
    public static void setSessionObject(String key,Object obj){
        SecurityUtils.getSubject().getSession(true).setAttribute(key,obj);
    }

    /**
     * 获取会话中对象
     * @param key
     * @return
     */
    public static Object getSessionAttribute(String key){
        Session session = SecurityUtils.getSubject().getSession(false);
        if(session != null){
            return (Object) session.getAttribute(key);
        }else{
            return null;
        }
    }
}
