package cn.estsh.i3plus.core.apiservice.auth.filter;

import cn.estsh.i3plus.core.apiservice.auth.AuthUtil;
import cn.estsh.i3plus.core.apiservice.configuration.ShiroAuthConfiguration;
import cn.estsh.i3plus.platform.common.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.platform.common.util.CommonConstWords;
import cn.estsh.i3plus.pojo.model.busi.ResultBean;
import cn.estsh.i3plus.pojo.platform.bean.SessionUser;
import com.alibaba.fastjson.JSONObject;
import org.apache.shiro.web.filter.AccessControlFilter;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.PrintWriter;

/**
 * @Description : 用户授权路径过滤
 * @Reference :
 * @Author : alwaysfrin
 * @CreateDate : 2018-10-13 19:17
 * @Modify:
 **/
public class UserFilter extends AccessControlFilter {
    public static final Logger LOGGER = LoggerFactory.getLogger(UserFilter.class);

    @Override
    protected boolean isAccessAllowed(ServletRequest servletRequest, ServletResponse response, Object mappedValue) throws Exception {
        SessionUser sessionUser = AuthUtil.getSessionUser();
        if (sessionUser != null) {
            LOGGER.info("===用户已登陆==={}",sessionUser);
            return true;
        } else {
            LOGGER.info("===用户未登陆==={}",sessionUser);
            return false;
        }
    }

    @Override
    protected boolean onAccessDenied(ServletRequest request, ServletResponse response) throws Exception {
        if (response instanceof HttpServletResponse) {
            LOGGER.info("===请求拒绝===请求路径：{}",((HttpServletRequest)request).getRequestURL());
            HttpServletResponse httpServletResponse = (HttpServletResponse)response;

            httpServletResponse.setHeader("Content-Type", "application/json;charset=UTF-8");
            httpServletResponse.setContentType("application/json;charset=UTF-8");
            PrintWriter out = null;

            try {
                ResultBean result = new ResultBean();
                result.setSuccess(false);
                result.setErrorMsg("未登录");
                result.setCode("need-login");
                result.setUrl(ShiroAuthConfiguration.USER_LOGIN_URI);

                out = response.getWriter();
                out.print(JSONObject.toJSONString(result));
                out.flush();
            } catch (IOException var11) {
                LOGGER.error(var11.getMessage(), var11);
            } finally {
                if (out != null) {
                    out.close();
                }

            }
        }

        return false;
    }
}
