package cn.estsh.i3plus.core.apiservice.configuration;

import cn.estsh.i3plus.core.apiservice.auth.ShiroSessionRedisDaoImpl;
import cn.estsh.i3plus.core.apiservice.auth.filter.AdminFilter;
import cn.estsh.i3plus.core.apiservice.auth.filter.SaAdminFilter;
import cn.estsh.i3plus.core.apiservice.auth.filter.UserFilter;
import cn.estsh.i3plus.core.apiservice.auth.realm.AdminAuthRealm;
import cn.estsh.i3plus.core.apiservice.auth.realm.DefaultModularRealm;
import cn.estsh.i3plus.core.apiservice.auth.realm.SaAuthRealm;
import cn.estsh.i3plus.core.apiservice.auth.realm.UserAuthRealm;
import cn.estsh.i3plus.platform.common.enumutil.CommonEnumUtil;
import cn.estsh.impp.framework.boot.configuration.RedisConfig;
import org.apache.shiro.authc.pam.ModularRealmAuthenticator;
import org.apache.shiro.cache.Cache;
import org.apache.shiro.cache.CacheManager;
import org.apache.shiro.cache.ehcache.EhCacheManager;
import org.apache.shiro.codec.Base64;
import org.apache.shiro.realm.Realm;
import org.apache.shiro.session.mgt.ExecutorServiceSessionValidationScheduler;
import org.apache.shiro.spring.web.ShiroFilterFactoryBean;
import org.apache.shiro.web.mgt.CookieRememberMeManager;
import org.apache.shiro.web.mgt.DefaultWebSecurityManager;
import org.apache.shiro.mgt.SecurityManager;
import org.apache.shiro.web.servlet.SimpleCookie;
import org.apache.shiro.web.session.mgt.DefaultWebSessionManager;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.boot.autoconfigure.AutoConfigureAfter;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.DependsOn;
import org.springframework.core.annotation.Order;

import javax.crypto.KeyGenerator;
import javax.crypto.SecretKey;
import javax.servlet.Filter;
import java.security.NoSuchAlgorithmException;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.Map;

/**
 * @Description : shiro权限配置
 * @Reference :
 * @Author : alwaysfrin
 * @CreateDate : 2018-10-13 18:25
 * @Modify:
 **/
@Configuration
@AutoConfigureAfter(RedisConfig.class)       //2类配置，延后加载
public class ShiroAuthConfiguration {
    public static final Logger LOGGER = LoggerFactory.getLogger(ShiroAuthConfiguration.class);

    //三种登陆请求
    public static String USER_LOGIN_URI = "/login";
    public static String USER_FILTER_URI = "/operate/*";
    public static String ADMIN_LOGIN_URI = "/adlogin";
    public static String ADMIN_FILTER_URI = "/adoerate/*";
    public static String SAADMIN_LOGIN_URI = "/salogin";
    public static String SAADMIN_FILTER_URI = "/saoerate/*";

    @Bean
    public AdminAuthRealm adminAuthRealm(){
        return new AdminAuthRealm();
    }

    @Bean
    public SaAuthRealm saAuthRealm(){
        return new SaAuthRealm();
    }

    @Bean
    public UserAuthRealm userAuthRealm(){
        return new UserAuthRealm();
    }

    public Map<String,Realm> supportRealmMap(){
        //支持的授权规则
        Map<String,Realm> realms = new HashMap();
        realms.put(CommonEnumUtil.USER_TYPE.USER.getCode(),userAuthRealm());
        realms.put(CommonEnumUtil.USER_TYPE.ADMIN.getCode(),adminAuthRealm());
        realms.put(CommonEnumUtil.USER_TYPE.SA.getCode(),saAuthRealm());

        return realms;
    }

    @Bean
    public ModularRealmAuthenticator modularRealmAuthenticator(){
        return  new DefaultModularRealm(supportRealmMap());
    }

    @Bean
    //@DependsOn("redisUtil")
    public SecurityManager securityManager(){
        LOGGER.info("【shiro-加载securityManager】");
        DefaultWebSecurityManager securityManager =  new DefaultWebSecurityManager();
        securityManager.setAuthenticator(modularRealmAuthenticator());
        securityManager.setRealms(supportRealmMap().values());
        securityManager.setCacheManager(getEhCacheManage());    //缓存管理
        securityManager.setSessionManager(getSessionManage());  //会话管理
        //securityManager.setRememberMeManager(getRememberManager());
        return securityManager;
    }

    /*@Bean
    public CookieRememberMeManager getRememberManager(){
        //rememberme cookie加密的密钥 建议每个项目都不一样 默认AES算法 密钥长度（128 256 512 位），通过以下代码可以获取
        //KeyGenerator keygen = KeyGenerator.getInstance("AES");
        //SecretKey deskey = keygen.generateKey();
        //System.out.println(Base64.encodeToString(deskey.getEncoded()));
        CookieRememberMeManager meManager = new CookieRememberMeManager();
        meManager.setCipherKey(Base64.decode("+vCeyY0qiyO1NdpOX9zX7w=="));
        meManager.setCookie(rememberMeCookie());
        return meManager;
    }

    *//**
     * 获取CipherKey
     *//*
    public static void main(String[] args){
        KeyGenerator keygen = null;
        try {
            keygen = KeyGenerator.getInstance("AES");
        } catch (NoSuchAlgorithmException e) {
            e.printStackTrace();
        }
        SecretKey deskey = keygen.generateKey();
        System.out.println(Base64.encodeToString(deskey.getEncoded()));
    }

    @Bean
    public SimpleCookie rememberMeCookie(){
        //这个参数是cookie的名称，对应前端的checkbox的name = rememberMe
        SimpleCookie simpleCookie = new SimpleCookie("impp_remember_me");
        //如果httyOnly设置为true，则客户端不会暴露给客户端脚本代码，使用HttpOnly cookie有助于减少某些类型的跨站点脚本攻击；
        simpleCookie.setHttpOnly(true);
        //记住我cookie生效时间,默认30天 ,单位秒：60 * 60 * 24 * 30
        simpleCookie.setMaxAge(259200);

        return simpleCookie;
    }*/

    @Bean
    public ShiroFilterFactoryBean shirFilter(SecurityManager securityManager) {
        ShiroFilterFactoryBean factoryBean = new ShiroFilterFactoryBean();
        factoryBean.setSecurityManager(securityManager);

        Map<String, Filter> filterMap = new HashMap();
        filterMap.put("user_filter", new UserFilter());
        filterMap.put("admin_filter", new AdminFilter());
        filterMap.put("saadmin_filter", new SaAdminFilter());
        factoryBean.setFilters(filterMap);

        Map<String, String> filterChain = new LinkedHashMap();
        //不需要过滤的路径，直接通过
        filterChain.put(USER_LOGIN_URI, "anon");
        filterChain.put(ADMIN_LOGIN_URI, "anon");
        filterChain.put(SAADMIN_LOGIN_URI, "anon");

        //需要过滤的路径
        filterChain.put(USER_FILTER_URI, "user_filter");
        filterChain.put(ADMIN_FILTER_URI, "admin_filter");
        filterChain.put(SAADMIN_FILTER_URI, "saadmin_filter");

        factoryBean.setFilterChainDefinitionMap(filterChain);

        LOGGER.info("【shiro过滤】加载完成...");
        return factoryBean;
    }

    /**
     * 缓存管理
     * @return
     */
    @Bean(name = "ehCacheManager")
    public CacheManager getEhCacheManage() {
        LOGGER.info("【shiro-加载缓存cacheManager】");
        EhCacheManager em = new EhCacheManager();
        em.setCacheManagerConfigFile("classpath:cache_session.xml");
        return em;
    }

    @Bean(name = "sessionManager")
    public DefaultWebSessionManager getSessionManage() {
        LOGGER.info("【shiro-加载sessionManager】");
        DefaultWebSessionManager sessionManager = new DefaultWebSessionManager();
        sessionManager.setSessionValidationScheduler(getExecutorServiceSessionValidationScheduler());
        sessionManager.setSessionValidationSchedulerEnabled(true);
        sessionManager.setDeleteInvalidSessions(true);
        sessionManager.setSessionIdCookieEnabled(true);
        sessionManager.setSessionIdCookie(getSessionIdCookie());
        sessionManager.setSessionDAO(getShiroSessionRedisDaoImpl());
        sessionManager.setGlobalSessionTimeout(30 * 60 * 1000); //30分钟有效期(以sessionDao中的有效时间为准)

        // 可以添加session 创建、删除的监听器
        return sessionManager;
    }

    @Bean
    public ShiroSessionRedisDaoImpl getShiroSessionRedisDaoImpl(){
        LOGGER.info("【shiro-加载缓存redisSessionManager】");
        ShiroSessionRedisDaoImpl shiroSessionRedisDao = new ShiroSessionRedisDaoImpl();

        LOGGER.info("【shiro-加载缓存cacheManager】");
        shiroSessionRedisDao.setLocalCacheManager(getEhCacheManage() );
        return shiroSessionRedisDao;
    }

    /**
     * 授权验证监听任务
     * @return
     */
    @Bean(name = "sessionValidationScheduler")
    public ExecutorServiceSessionValidationScheduler getExecutorServiceSessionValidationScheduler() {
        LOGGER.info("【shiro-加载sessionValidationScheduler】");
        ExecutorServiceSessionValidationScheduler scheduler = new ExecutorServiceSessionValidationScheduler();
        scheduler.setInterval(3 * 60 * 1000);   //每3分钟执行一次验证
        return scheduler;
    }

    @Bean(name = "sessionIdCookie")
    public SimpleCookie getSessionIdCookie() {
        LOGGER.info("【shiro-加载sessionIdCookie】");
        SimpleCookie cookie = new SimpleCookie("sid");
        cookie.setHttpOnly(true);
        cookie.setMaxAge(-1);

        return cookie;
    }
}
