package cn.estsh.i3plus.core.apiservice.controller;

import cn.estsh.i3plus.core.api.iservice.busi.ISystemResourceService;
import cn.estsh.i3plus.core.apiservice.mq.I3CoreQueue;
import cn.estsh.i3plus.platform.common.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.factory.bean.MachineFactory;
import cn.estsh.i3plus.pojo.model.busi.ResultBean;
import cn.estsh.i3plus.pojo.platform.bean.SysLocaleLanguage;
import cn.estsh.i3plus.pojo.platform.bean.SysLocaleResource;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.exception.ImppExceptionEnum;
import cn.estsh.impp.framework.boot.util.ImppRedis;
import com.rabbitmq.client.Channel;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.rabbit.connection.CachingConnectionFactory;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.io.IOException;
import java.util.Locale;
import java.util.concurrent.TimeoutException;

/**
 * @Description :
 * @Reference :
 * @Author : alwaysfrin
 * @CreateDate : 2018-09-26 10:34
 * @Modify:
 **/
@RestController
@RequestMapping("/demo-system")
@Api(description="系统服务demo")
public class DemoSystemController {
    private static final Logger LOGGER = LoggerFactory.getLogger(DemoSystemController.class);

    /**
     * 缓存
     */
    @Resource(name="redisUtil")
    private ImppRedis imppRedis;

    /**
     * rabbitMQ队列
     */
    @Autowired
    private RabbitTemplate rabbitTemplate;

    @Autowired
    private ISystemResourceService systemResourceService;

    @PutMapping(value="/addlanguage")
    @ApiOperation(value="添加语言")
    public ResultBean addlanguage(SysLocaleLanguage lang) {
        systemResourceService.saveSysLocaleLanguage(lang);
        return new ResultBean(true,"操作成功");
    }

    @PutMapping(value="/listlang")
    @ApiOperation(value="查询语言")
    public ResultBean listlang(SysLocaleLanguage lang) {
        return new ResultBean(true,"操作成功",systemResourceService.listSysLocaleLanguage(lang));
    }

    @PutMapping(value="/addres")
    @ApiOperation(value="添加资源")
    public ResultBean addres(SysLocaleResource res) {
        systemResourceService.saveSysLocaleResource(res);
        return new ResultBean(true,"操作成功");
    }

    @PutMapping(value="/listres")
    @ApiOperation(value="查询资源")
    public ResultBean listres(SysLocaleResource res) {
        return new ResultBean(true,"操作成功",systemResourceService.listSysLocaleResource(res));
    }

    @GetMapping(value="/getlocaleresource")
    @ApiOperation(value="语言资源匹配",notes="资源配置文件及国际化")
    public ResultBean getlocaleresource(int resourceType,String langCode,String resKey) {
        Locale locale = LocaleContextHolder.getLocale();
        System.out.println("当前语言："+locale.getLanguage() + "，国家：" + locale.getCountry());

        SysLocaleResource res = systemResourceService.getSysLocaleResourceByLanguage(resourceType,langCode,resKey);
        System.out.println("获取的资源：" + res);

        return new ResultBean(true,"操作成功",res);
    }
}
