package cn.estsh.i3plus.core.apiservice.serviceimpl.busi;

import cn.estsh.i3plus.core.api.iservice.busi.ISystemResourceService;
import cn.estsh.i3plus.platform.common.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.model.common.Pager;
import cn.estsh.i3plus.pojo.model.common.PagerHelper;
import cn.estsh.i3plus.pojo.platform.bean.SysLocaleLanguage;
import cn.estsh.i3plus.pojo.platform.bean.SysLocaleResource;
import cn.estsh.i3plus.pojo.platform.repository.SysLocaleLanguageRepository;
import cn.estsh.i3plus.pojo.platform.repository.SysLocaleResourceRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Sort;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.List;

/**
 * @Description : 系统资源服务
 * @Reference :
 * @Author : alwaysfrin
 * @CreateDate : 2018-10-11 17:00
 * @Modify:
 **/
@Service
public class SystemResourceService implements ISystemResourceService {

    @Autowired
    private SysLocaleLanguageRepository sysLocaleLanguageRDao;

    @Autowired
    private SysLocaleResourceRepository sysLocaleResourceRDao;

    @Override
    public SysLocaleLanguage saveSysLocaleLanguage(SysLocaleLanguage lang) {
        return sysLocaleLanguageRDao.insert(lang);
    }

    @Override
    public SysLocaleLanguage updateSysLocaleLanguage(SysLocaleLanguage lang) {
        return sysLocaleLanguageRDao.save(lang);
    }

    @Override
    public void deleteSysLocaleLanguageById(long id) {
        sysLocaleLanguageRDao.deleteById(id);
    }

    @Override
    public List<SysLocaleLanguage> listSysLocaleLanguage(SysLocaleLanguage lang) {
        if(lang == null){
            return sysLocaleLanguageRDao.list();
        }else {
            return sysLocaleLanguageRDao.findByProperty(
                    new String[]{"languageCode", "languageName", "isDefault"},
                    new Object[]{lang.getLanguageCode(), lang.getLanguageName(), lang.getIsDefault()});
        }
    }

    @Override
    public List<SysLocaleLanguage> listSysLocaleLanguageByPager(SysLocaleLanguage lang, Pager pager) {
        if(lang == null) {
            pager = PagerHelper.getPager(pager, sysLocaleLanguageRDao.listCount());
            return sysLocaleLanguageRDao.listPager(pager);
        }else {
            String[] params = new String[]{"languageCode", "languageName", "isDefault"};
            Object[] values = new Object[]{lang.getLanguageCode(), lang.getLanguageName(), lang.getIsDefault()};

            pager = PagerHelper.getPager(pager, sysLocaleLanguageRDao.findByPropertyCount(params,values));
            return sysLocaleLanguageRDao.findByPropertyPage(params,values,"order by id asc",pager);
        }
    }

    @Override
    public SysLocaleResource saveSysLocaleResource(SysLocaleResource res) {
        return sysLocaleResourceRDao.insert(res);
    }

    @Override
    public SysLocaleResource updateSysLocaleResource(SysLocaleResource res) {
        return sysLocaleResourceRDao.save(res);
    }

    @Override
    public void deleteSysLocaleResourceById(long id) {
        sysLocaleResourceRDao.deleteById(id);
    }

    @Override
    public List<SysLocaleResource> listSysLocaleResourceByPager(SysLocaleResource res, Pager pager) {
        if(res == null){
            int count = sysLocaleResourceRDao.listCount();
            pager = PagerHelper.getPager(pager,count);
            return sysLocaleResourceRDao.listPager(pager);
        }else {
            String[] params = new String[]{"resourceType", "languageCode", "resourceKey", "resourceValue", "isSystem", "isDefault"};
            Object[] values = new Object[]{res.getResourceType(), res.getLanguageCode(), res.getResourceKey(), res.getResourceValue(), res.getIsSystem(), res.getIsDefault()};

            pager = PagerHelper.getPager(pager,sysLocaleResourceRDao.findByPropertyCount(params,values));
            return sysLocaleResourceRDao.findByPropertyPage(params,values,"order by id asc", pager);
        }
    }

    @Override
    public List<SysLocaleResource> listSysLocaleResource(SysLocaleResource res) {
        if(res == null){
            return sysLocaleResourceRDao.list();
        }else {
            return sysLocaleResourceRDao.findByProperty(
                    new String[]{"resourceType", "languageCode", "resourceKey", "resourceValue", "isSystem", "isDefault"},
                    new Object[]{res.getResourceType(), res.getLanguageCode(), res.getResourceKey(), res.getResourceValue(), res.getIsSystem(), res.getIsDefault()});
        }
    }

    @Override
    public SysLocaleResource getSysLocaleResourceByLanguage(int resourceType,String langCode,String resKey) {
        List<SysLocaleResource> resList = sysLocaleResourceRDao.findByProperty(
                new String[]{"resourceType", "languageCode", "resourceKey"},
                new Object[]{resourceType,langCode, resKey});

        if(CollectionUtils.isEmpty(resList)){
            //不存在，则使用默认的
            resList = sysLocaleResourceRDao.findByProperty(
                    new String[]{"resourceType", "resourceKey", "isDefault"},
                    new Object[]{resourceType,resKey, CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue()});
        }
        if(resList.size() > 0) {
            return resList.iterator().next();
        }else{
            return null;
        }
    }
}
